<?php

namespace Drupal\trackmyadmin\Utility;

use Drupal\Core\Database\Connection;
use Drupal\user\Entity\User;

/**
 * Defines a Helper for all the queries.
 */
class QueryHelper {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Constructs the QueryHelper Object.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   */
  public function __construct(Connection $connection) {
    $this->connection = $connection;
  }

  /**
   * Helper function to handle database insertions.
   *
   * @param string $table
   *   The name of the table in which data to be inserted.
   * @param array $values
   *   Array of values corresponding to the table.
   *
   * @return int
   *   Reference of the insertion.
   */
  public function insertValues($table, array $values) {
    if ($table == 'trackmyadmin_users') {
      $inserted = $this->connection->insert($table)
        ->fields([
          'uid' => $values['uid'],
          'name' => $values['name'],
          'team' => $values['team'],
          'date' => $values['date'],
          'ip' => $values['ip'],
          'location' => $values['location'],
        ])
        ->execute();
    }
    if ($table == 'trackmyadmin_activity') {
      $inserted = $this->connection->insert($table)
        ->fields([
          'id' => $values['id'],
          'field' => $values['field'],
          'url' => $values['url'],
          'value' => $values['value'],
          'modified_type' => $values['modified_type'],
          'date' => $values['date'],
        ])
        ->execute();
    }
    return $inserted;
  }

  /**
   * Helper function to handle database Fetch.
   *
   * @param string $type
   *   The type of the data as per need.
   * @param array $params
   *   Array of values corresponding to the type.
   *
   * @return array
   *   Array of data requested.
   */
  public function fetchValues($type, array $params = []) {
    $data = [];
    if ($type == 'content_config') {
      $content_results = $this->connection->query("SELECT COUNT(*) FROM {trackmyadmin_activity} WHERE modified_type LIKE 'content'")->fetchField();
      $config_results = $this->connection->query("SELECT COUNT(*) FROM {trackmyadmin_activity} WHERE modified_type LIKE 'config'")->fetchField();
      // Make sure no division by 0.
      if (($config_results + $content_results) > 0) {
        $data['config'] = round($config_results / ($config_results + $content_results), 2) * 100;
        $data['content'] = round($content_results / ($config_results + $content_results), 2) * 100;
      }
      else {
        $data['config'] = $data['content'] = 0;
      }
      $user_results = $this->connection->query("SELECT DISTINCT(id), date FROM {trackmyadmin_activity} ORDER BY date DESC")->fetchAll();
      if (!empty($user_results)) {
        foreach ($user_results as $user_result) {
          // Use this helper to fetch more details specific to user.
          $user_item = $this->recordsbyField('trackmyadmin_users', 'id', $user_result->id);
          if (!empty($user_item)) {
            $data['users'][$user_result->id]['name'] = $user_item[0]->name;
            $data['users'][$user_result->id]['team'] = $user_item[0]->team;
            $user = User::load($user_item[0]->uid);
            $data['users'][$user_result->id]['user'] = $user->getAccountName();
            if ($user_item[0]->date) {
              $data['users'][$user_result->id]['date'] = \Drupal::service('date.formatter')->format($user_item[0]->date, 'custom', 'm-d-Y H:i:s');
            }
          }
        }
      }
    }
    if (($type == 'name' || $type == 'team' || $type == 'uid')  && isset($params['data'])) {
      $results = $this->recordsbyField('trackmyadmin_users', $type, $params['data']);
      if (!empty($results)) {
        foreach ($results as $key => $value) {
          $data['data'][$key]['name'] = $value->name;
          $data['data'][$key]['team'] = $value->team;
          $user = User::load($value->uid);
          $data['data'][$key]['user'] = $user->getAccountName();
          if ($value->date) {
            $data['data'][$key]['date'] = \Drupal::service('date.formatter')->format($value->date, 'custom', 'm-d-Y H:i:s');
          }
          $data['data'][$key]['ip'] = $value->ip;
          $data['data'][$key]['location'] = $value->location;
        }
      }
    }
    if ($type == 'content' || $type == 'config') {
      $results = $this->recordsbyField('trackmyadmin_activity', 'modified_type', $type);
      if (!empty($results)) {
        foreach ($results as $key => $value) {
          $user_item = $this->recordsbyField('trackmyadmin_users', 'id', $value->id);
          if (!empty($user_item)) {
            $data['data'][$key]['name'] = $user_item[0]->name;
            $data['data'][$key]['field'] = $value->field;
            $data['data'][$key]['url'] = $value->url;
            $data['data'][$key]['value'] = $value->value;
            if ($value->date) {
              $data['data'][$key]['date'] = \Drupal::service('date.formatter')->format($value->date, 'custom', 'm-d-Y H:i:s');
            }
          }
        }
      }
    }
    return $data;
  }

  /**
   * Helper function for database fetch.
   *
   * @param string $table
   *   The name of the table in which data to be fetched.
   * @param string $field
   *   The name of the field corresponding to the table above.
   * @param string $field_value
   *   The value corresponding to the field above.
   *
   * @return array
   *   Array of data requested.
   */
  public function recordsbyField($table, $field, $field_value) {
    return $this->connection->query("SELECT * FROM {" . $table . "} WHERE " . $field . " LIKE :field_value ORDER BY date DESC", [':field_value' => $field_value])->fetchAll();
  }

}
